package com.example.greenwatch;

import android.annotation.SuppressLint;
import android.content.Context;
import android.widget.Toast;

import java.io.IOException;
import java.net.DatagramPacket;
import java.net.DatagramSocket;
import java.net.Inet4Address;
import java.net.InetAddress;
import java.net.NetworkInterface;
import java.net.SocketException;
import java.net.UnknownHostException;
import java.util.Enumeration;

public class WifiCommunication {
    private final MainActivity mainActivity;
    private final InetAddress address;
    private final int port;
    private final Context context;
    private boolean newMsg;
    private String rxString;
    private String sendMsg;
    //private String sendMsg = "default";
    private final DatagramSocket socket;
    volatile private boolean running;
    @SuppressLint("SetTextI18n")
    public WifiCommunication(MainActivity mainActivity, Context context, int port) {
        this.mainActivity = mainActivity;
        this.port = port;
        this.context = context;
        try {
            socket = new DatagramSocket(this.port);
            socket.setBroadcast(true);
            address = InetAddress.getByName("255.255.255.255");
            running = true;
            new ReceiveThread().start();
            new SendThread().start();
        } catch (SocketException | UnknownHostException e) {
            throw new RuntimeException(e);
        }
        mainActivity.runOnUiThread(() -> Toast.makeText(context,"Communication running", Toast.LENGTH_SHORT).show());
    }
    private class ReceiveThread extends Thread {
        private String rxString = "";
        private String previousRxString = "";
        @Override
        public void run() {
            try {
                do {

                    byte[] rxBuffer = new byte[1024];
                    DatagramPacket packet = new DatagramPacket(rxBuffer, rxBuffer.length);
                    socket.receive(packet);
                    rxString = new String(packet.getData(), 0, packet.getLength());

                    if(!previousRxString.equals(rxString))
                        mainActivity.runOnUiThread(() -> mainActivity.tvReceivedData.setText(rxString));

                    previousRxString = rxString;

                } while (running);
            }
            catch (IOException e) {
                e.printStackTrace();
            }
        }
    }

    public String showMessage(){
        return rxString;
    }
    public void setMessage(String message, boolean newMsg){
        this.sendMsg=message;
        this.newMsg = newMsg;
    }
    private class SendThread extends Thread {
        private int tmpCnt = 0;
        @Override
        public void run() {
            try {
                do {

                    if(newMsg == true)
                    {
                        newMsg = false;

                        String txString = getLocalIpAddress() + " sends. #" + tmpCnt++ + sendMsg;
                        byte[] txBuffer = txString.getBytes();

                        DatagramPacket txPacket = new DatagramPacket(txBuffer, txBuffer.length, address, port);

                        for(int i = 0; i < 20; i++) {
                            socket.send(txPacket);
                        }
                    }
                } while (running);
            } catch (IOException e) {
                e.printStackTrace();
            }
        }
    }
    public static String getLocalIpAddress() {
        try {
            for (Enumeration<NetworkInterface> en = NetworkInterface.getNetworkInterfaces(); en.hasMoreElements();) {
                NetworkInterface networkInterface = (NetworkInterface) ((Enumeration<?>) en).nextElement();
                for (Enumeration<InetAddress> addresses = networkInterface.getInetAddresses(); addresses.hasMoreElements();) {
                    InetAddress inetAddress = addresses.nextElement();
                    if (!inetAddress.isLoopbackAddress() && inetAddress instanceof Inet4Address) {
                        return inetAddress.getHostAddress();
                    }
                }
            }
        } catch (SocketException ex) {
            ex.printStackTrace();
        }
        return null;
    }
    public void stopCommunication() {
        running = false;
        socket.close();
    }
}
